////////////////////////////////////////////////////////////
// 共通実装
////////////////////////////////////////////////////////////

//------------------------------------------
// オブジェクト
//------------------------------------------

// オブジェクトのベースクラス
// ※要継承
class Entity {
  constructor() {}

  static getLabels() {
    throw new Error("Method 'getLabels' must be implemented");
  }

  static getDisplayLabels() {
    throw new Error("Method 'getDisplayLabels' must be implemented");
  }

  static getSearchCellIndexes() {
    throw new Error("Method 'getSearchCellIndexes' must be implemented");
  }

  static getUniqueKeyIndex() {
    throw new Error("Method 'getUniqueKeyIndex' must be implemented");
  }

  static getActionMenus() {
    throw new Error("Method 'getActionMenus' must be implemented");
  }

  toArray() {
    throw new Error("Method 'toArray' must be implemented");
  }
}

// オブジェクト群のベースクラス
// ※要継承
class Entities {
  ITEMS_PER_PAGE = 30;
  static ORDER_ASC = 'asc';
  static ORDER_DESC = 'desc'

  constructor() {
    this._entities = [];
    this._displayEntities = [];
    this._sortIndex = 0;
    this._order = Entities.ORDER_ASC;
    this._currentPage = 1;
    this._keyRegex = '';
    this._searchCellIndexes = [];
  }

  setSortIndex(index) {
    this._sortIndex = index;
  }

  setOrder(order) {
    this._order = order;
  }

  setCurrentPage(page) {
    this._currentPage = page
  }

  getCurrentPage() {
    return this._currentPage;
  }

  intiCurrentPage() {
    this._currentPage = 1;
  }

  setKeyRegex(key) {
    this._keyRegex = key;
  }

  initKeyRegex() {
    this._keyRegex = '';
  }

  setSearchCellIndexes(searchCellIndexes) {
    this._searchCellIndexes = searchCellIndexes;
  }

  // 全データを取得
  getAll() {
    return this._entities;
  }

  // 全表示データを取得
  get() {
    this.filterByKey();
    this.sortDisplayEntities();
    return this._displayEntities;
  }

  // 現在のページの表示データのみを取得
  getByPage() {
    this.filterByKey();
    this.sortDisplayEntities();
    const startIndex = (this._currentPage - 1) * this.ITEMS_PER_PAGE;
    const endIndex = startIndex + this.ITEMS_PER_PAGE;
    return this._displayEntities.slice(startIndex, endIndex);
  }

  // 全Entity配列から指定したindexの値が一致するデータを探索し返却
  find(index, value) {
    return this._entities.find(entity => entity[index] === value) || null;
  }

  // データを追加
  add(entity) {
    this._entities.push(entity);
  }

  update(oleEntity, newEntity) {
    this.delete(oleEntity);
    this.add(newEntity);
  }

  // 引数と一致するデータを削除
  delete(entity) {
    this._entities = this._entities.filter(item => !Common.isEqualArrays(item, entity));
  }

  // 表示データをリセット
  resetDisplayEntities() {
    this._displayEntities = this._entities
  }

  // 表示データをソート
  sortDisplayEntities() {
    if (this._order === Entities.ORDER_ASC) {
      this.sortDisplayEntitiesAsc();
    } else {
      this.sortDisplayEntitiesDesc();
    }
  }

  // 表示データを指定したインデックスで昇順ソートする
  sortDisplayEntitiesAsc() {
    this._displayEntities = this._displayEntities.sort(function (a, b) {
      const valA = a[this._sortIndex];
      const valB = b[this._sortIndex];
      return String(valA).localeCompare(String(valB));
    }.bind(this));
  }

  // 表示データを指定したインデックスで降順ソートする
  sortDisplayEntitiesDesc() {
    this._displayEntities = this._displayEntities.sort(function (a, b) {
      const valA = a[this._sortIndex];
      const valB = b[this._sortIndex];
      return String(valB).localeCompare(String(valA));
    }.bind(this));
  }

  // 表示データをキーワードでフィルタする
  filterByKey() {
    let searchCellIndexes;
    // デフォルトは各行のすべてのセルが検索対象
    if (this._searchCellIndexes.length === 0) {
      searchCellIndexes = Array.from({ length: this._entities[0].length }, (_, i) => i);
    } else {
      searchCellIndexes = this._searchCellIndexes;
    }

    // 指定したセルの文字列（小文字）を結合して検索ワード（小文字）の部分一致でフィルターし、表示データに格納する
    this._displayEntities = this._entities.filter(function (entity) {
      // 文字列を小文字に変換し結合
      const concatenatedVal = searchCellIndexes.map(function (index) {
        return entity[index];
      }).join('');
      // keyに.*が含まれる場合はそのまま、それ以外は*を.*に変換
      const regexString = this._keyRegex.includes('.*') ? this._keyRegex : this._keyRegex.replace(/\*/g, '.*');
      // キーワードを含むデータにフィルター
      const regex = new RegExp(regexString, 'i'); // 'i' フラグは大文字小文字を区別しないフラグ
      return regex.test(concatenatedVal);
      // return concatenatedVal.toLowerCase().includes(key);
    }.bind(this));
  }
}

//------------------------------------------
// API
//------------------------------------------

class Api {
  // get function api url
  static getFunctionURL(subFuncUrl) {
    throw new Error("Method 'getFunctionURL' must be implemented");
  }

  static async getAuthToken() {
    let authToken;
    if('LOCAL_URL' in ENV) {
      // for local dev
      authToken = 'dummy';
    } else {
      authToken = await window.auth.getAuthenticationToken();
    }
    return authToken;
  }

  static async all(authToken) {
    throw new Error("Method 'all' must be implemented");
  }

  static async get(authToken, id) {
    throw new Error("Method 'get' must be implemented");
  }

  static async create(authToken) {
    throw new Error("Method 'create' must be implemented");
  }

  static async update(authToken) {
    throw new Error("Method 'update' must be implemented");
  }

  static async delete(authToken) {
    throw new Error("Method 'delete' must be implemented");
  }
}

  // 以下のプロパティを持つErrorオブジェクトを生成する。
  //   field:
  //     process           : ALWAYS   : process name     e.g. 'token', 'function'
  //     httpStatus        : ALWAYS   : httpStatus       e.g. 400, 500
  //                                    0: no httpStatus e.g. jQuery.ajax() connection error, internal error
  //     funcResponseJson  : OPTIONAL : error info from function api as json format
  //     detail            : OPTIONAL : error information text
  //     innerError        : ALWAYS   : original error data 
  class ApiError extends Error {
    static HTTPSTATUS_INVALID = 0;

    httpStatus = ApiError.HTTPSTATUS_INVALID;
    funcResponseJson = null
    detail = "";
    msgLevel = "";

    constructor(error, process) {
      super('');  // Errorクラスがmessageプロパティを持っているが、使用しないため空文字列を入れる
      this.name = this.constructor.name;  // 自身のクラス名をセットする
      this.process = process;
      this.innerError = error;
      this.parseErrorInternal(error);  // httpStatus, funcResponseJson, innerError をセット
    }

    // オリジナルのErrorオブジェクトを解析して、このErrorインスタンスのプロパティに値をセット
    parseErrorInternal(error) {
      if ( ('status' in error) && ('responseText' in error) && (error.responseText != undefined)) {
        // maybe jqXHR and has responseBody
        this.httpStatus = error.status;
        try {
          // function api error json ?
          this.funcResponseJson = JSON.parse(error.responseText);
        } catch {
          this.detail = error.responseText;
        }

      } else {
        // maybe jQuery.ajax() error
        // maybe internal error
        this.httpStatus = ApiError.HTTPSTATUS_INVALID;  // no status detail

        if (error instanceof Error) {
          // maybe internal error
          this.detail = error.message;
        }
      }
    }
  }

  // 各画面・機能で使用するApiErrorクラス
  class AuthApiError extends ApiError {
    constructor(error) {
      super(error, 'auth');
    }
  }

//------------------------------------------
// フォーム・バリデーション
//------------------------------------------

// フォーム関連の一連の処理を行うクラス
// ※要継承
class FormHandler {
  _handleMap = []

  constructor() {
    // フォームデータと失敗結果を初期化
    this._formData = {};
    this._failedResults = [];
  }

  // フォームがサブミットされたときの処理
  async handleFormSubmit() {
    this._getFormData();
    this._validateFormData();
    // 失敗結果がある場合は送信を中止
    if (this._failedResults.length !== 0) {
      return false;
    }
    this._sanitizeFormData();
    this._addCustomFields();
    // サーバーにデータを送信
    return await this.sendDataToServer();
  }

  // フォームの値を取得してインスタンス変数に格納
  _getFormData() {
    this._formData = {};
    this._handleMap.forEach((item) => {
      const value = this._getInputValue(item.formId);
      // 入力された値をフォームデータに格納
      this._formData[item.key] = value;
    });
  }

  // 各フォームの値を取得
  _getInputValue(formId) {
    const element = $(`#${formId}`);
    if (element.is(':checkbox')) {
      return $(`#${formId}`).prop('checked');
    } else {
      return $(`#${formId}`).val().trim();
    }
  }

  // フォームデータのバリデーションを実行
  _validateFormData() {
    const failedResults = []
    this._handleMap.forEach((item) => {
      const validateResult = item.validator.validate(this._formData[item.key]);

      // バリデーションに失敗した場合、エラー結果を追加
      if (validateResult && !validateResult.result) {
        failedResults.push({
          result: validateResult.result,
          displayLabel: item.displayLabel,
          message: validateResult.message
        });
      }
    });

    // 失敗した結果をインスタンス変数に格納
    this._failedResults = failedResults;
  }

  // フォームデータのサニタイズを実行
  _sanitizeFormData() {
    for (const key in this._formData) {
      if (this._formData.hasOwnProperty(key)) {
        this._formData[key] = this._sanitize(this._formData[key]);
      }
    }
  }

  // サニタイズ処理
  _sanitize(value) {
    if (typeof value === 'boolean') {
      return value;
    }
    // サニタイズの必要性が出てきたら適宜処理を行う（現状必要な処理はない）
    // 画面固有の処理の場合、オーバーライドして実装すること。
    return value;
  }

  getFailedResultsFirst() {
    if (this._failedResults && this._failedResults.length > 0) {
      return this._failedResults[0];
    } else {
      return {
        result: false,
        displayLabel: undefined,
        message: '失敗しました。'
      }
    }
  }

  _addCustomFields() {
    // フォームの入力値とは別に何かデータを追加したい場合に使用する。
  }

  async sendDataToServer() {
    // サーバーへデータを送信する処理を実装
    throw new Error("Method 'sendDataToServer' must be implemented");
  }
}

// Validatorクラス
class Validator {
  constructor() {
    this.rules = [];
  }

  _addRule(ruleFunction) {
    this.rules.push(ruleFunction);
  }

  validate(value) {
    let ruleResult;
    for (const rule of this.rules) {
      ruleResult = rule(value);
      if (!ruleResult.result) {
        break;
      }
    }
    return ruleResult;
  }

  _resultSuccess() {
    return {
      result: true,
      message: 'OK'
    }
  }

  _resultFailed(message) {
    return {
      result: false,
      message: message
    }
  }

  // ユーザー名はmin文字以上
  _minLength(value, min) {
    if (value.length >= min) {
      return this._resultSuccess();
    }
    const message = `${min}文字以上で入力してください。`;
    return this._resultFailed(message);
  }

  // ユーザー名はmax文字以下
  _maxLength(value, max) {
    if (value.length <= max) {
      return this._resultSuccess();
    }
    const message = `${max}文字以下で入力してください。`;
    return this._resultFailed(message);
  }

  // パスワード（確認用）が一致するか
  _rePasswordIsEqual(value, rePasswordFieldId) {
    if (value === $(`#${rePasswordFieldId}`).val().trim()) {
      return this._resultSuccess();
    }
    const message = `パスワードが一致しません。`;
    return this._resultFailed(message);
  }
}

// バリデーションが不要なフォーム用
class NothingValidator extends Validator {
  constructor() {
    super();
  }
}

//------------------------------------------
// 画面描画
//------------------------------------------

// 画面描画クラス（静的）
// ※必要に応じて継承し、オーバーライド・拡張して使用
// ※他画面で共通して使用可能な関数はこちらに追加する
// ※単純な画面描画のみを行うようにし、ロジックは極力持たないようにする
class ViewRenderer {
  // テーブルの内容をクリアする
  static clearTbody(tableId) {
    $(`#${tableId} tbody`).empty();
  }

  // テーブルのヘッダー（データ列）を描画する
  static renderThead(tableId, labels, displayLabels) {
    // ヘッダーのデータ列を追加する
    for (let i = 0; i < labels.length; i++) {
      $(`#${tableId} thead tr`).append(`
        <th scope="col" aria-readonly="true" class="table-sort-enabled" data-order="" data-index=${i}>
          <a href="#" role="button" aria-label="${labels[i]}" tabindex="0">
            ${displayLabels[i]}
            <span class="sort-icon"></span>
          </a>
        </th>
      `);
    }
  }

  // テーブルのヘッダー（アクション列）を描画する
  static renderTheadAddition(tableId) {
    // ヘッダーにアクション列を追加する
    $(`#${tableId} thead tr`).append(`
      <th scope="col" aria-readonly="true" style="width:6%;" class="sort-disabled" aria-label="Actions" data-th="<span class=\'sr-only\'>Actions</span>">
        <span class="sr-only">Actions</span>
      </th>
    `);
  }

  // テーブルのボディを描画する
  static renderTbody() {
    // TODO: 各ページ異なる実装になることが想定されるため共通化はしない。継承クラスを作成し各ページで実装する。
    throw new Error("Method 'renderTbody' must be implemented");
  }

  // テーブルの読み込み中ボディを描画する
  static renderLoadingTbody(tableId, colspanNum) {
    $(`#${tableId} tbody`).append(`
      <tr>
        <td align="center" colspan="` + colspanNum + `">
          <span class="fa fa-spinner fa-spin" style="margin-right: 5px;" aria-hidden="true"/>
          読み込み中...
        </td>
      </tr>
    `);
  }

  // テーブル行にアクションメニューを描画する
  static renderActionMenuCell(targetDom, menus) {
    // アクションメニューセルを追加
    targetDom.append(`
      <td aria-label="アクション メニュー">
        <div class="dropdown action">
          <button class="btn btn-default btn-xs aria-exp" data-toggle="dropdown" aria-expanded="false" aria-label="アクション メニュー" title="アクション メニュー">
              <span class="fa fa-chevron-circle-down fa-fw" aria-hidden="true"></span>
          </button>
          <ul class="dropdown-menu" role="menu">
          </ul>
        </div>
      </td>
    `);
    for (let i = 0; i < menus.length; i++) {
      targetDom.find('ul').append(`
        <li role="none">
            <a class="${menus[i].class}" role="menuitem" tabindex="-1" href="#" title="削除">
                <span class="" aria-hidden="true"></span>
                ${menus[i].title}
            </a>
        </li>  
      `)
    }
  }

  static renderPagination(itemsLength, currentPage, perPage) {
    $('#pagination').empty();
    // ページネーション不要
    if (itemsLength <= perPage) {
      return
    }
    // ページネーション描画
    const totalPages = Math.ceil(itemsLength / perPage);
    ViewRenderer.renderPaginationPrev(currentPage);
    ViewRenderer.renderPaginationItems(currentPage, totalPages);
    ViewRenderer.renderPaginationNext(currentPage, totalPages);
  }

  static renderPaginationPrev(currentPage) {
    $('#pagination').append(`
      <li class="page-item ${currentPage === 1 ? 'disabled' : ''}" id="previousPage">
        <a class="entity-pager-prev-link" href="#" aria-label="Previous">
          <span aria-hidden="true">&laquo;</span>
        </a>
      </li>
    `);
  }

  static renderPaginationItems(currentPage, totalPages) {
    
    for (let i = 1; i <= totalPages; i++) {
      $('#pagination').append(`
        <li class="page-item ${currentPage === i ? 'active' : ''}" data-page="${i}">
          <a class="page-link" href="#">${i}</a>
        </li>
      `);
    }
  }

  static renderPaginationNext(currentPage, totalPages) {
    $('#pagination').append(`
      <li class="page-item ${currentPage === totalPages ? 'disabled' : ''}" id="nextPage">
        <a class="entity-pager-prev-link" href="#" aria-label="Next">
          <span aria-hidden="true">&raquo;</span>
        </a>
      </li>
    `);
  }

  static updateSortDom(targetDom, order) {
    // 全てのソートアイコンを一旦リセット
    $('.table-sort-enabled').data('order', null).find('.sort-icon').html('');
    // ソートアイコンを更新
    if (order === Entities.ORDER_ASC) {
      targetDom.find('.sort-icon').html('<i class="fa fa-arrow-up"></i>');
    } else {
      targetDom.find('.sort-icon').html('<i class="fa fa-arrow-down"></i>');
    }
    // ソート順データを更新
    targetDom.data('order', order);
  }

  static renderValidateMessage(selector, message) {
    ViewRenderer.resetValidateMessage();
    $(`${selector}`).find('.validation-summary').css('display', 'block');
    $(`${selector}`).find('.validation-summary').append(`
      <h4 class="validation-header" role="none">
        <span role="presentation" class="fa fa-info-circle"></span>
        以下の理由により、フォームを送信できませんでした。
      </h4>
      <ul role="presentation">
        <li>${message}</li>
      </ul>
    `);
  }

  static resetValidateMessage() {
    $('.validation-summary').css('display', 'none');
    $('.validation-summary').empty();
  }

  static showAlert(alertType, message) {
    // Validate alert type
    if (!['success', 'info', 'warning', 'danger'].includes(alertType)) {
      console.error('Invalid alert type');
      return;
    }

    // 使用するアラートクラスを作成
    const alertClass = 'alert-' + alertType;
    // dangerの場合、文字列をerrorに変換する
    if (alertType === 'danger') {
      alertType = 'error';
    }

    // アラートタイプの最初の1文字を大文字に変換
    const capitalizedFirstLetter = alertType.charAt(0).toUpperCase() + alertType.slice(1);

    // 挿入するアラートElement
    const alertElement = $(`
      <div class="alert ${alertClass} alert-dismissible" role="alert">
        <button type="button" class="close" data-dismiss="alert" aria-label="Close">
          <span aria-hidden="true">&times;</span>
        </button>
        <strong>${capitalizedFirstLetter}: </strong> ${message}
      </div>
    `);

    // Elementの追加
    $('#alertMessage').append(alertElement);

    // error or warning以外の場合、5秒後に自動で削除
    if (alertType !== 'error' && alertType !== 'warning') {
      setTimeout(function () {
        alertElement.alert('close');
      }, 5000);
    }
  }

  // モーダルを表示
  static showModal(selector) {
    $(`${selector}`).modal('show');
    $(`${selector}`).attr('aria-hidden', 'false');
    $('.modal').find('.form-control').val('');
  }

  // モーダルを閉じる処理
  static closeModal() {
    $('.modal').modal('hide');
    $('.modal').attr('aria-hidden', 'true');
    $('.modal-message').empty();
    ViewRenderer.resetValidateMessage();
  }

  // 編集モーダルにメッセージを描画する
  static setModalEditMessage(message) {
     $('#edit-message').text(message);
  }

  // 削除モーダルにメッセージを描画する
  static setModalDeleteMessage(message) {
     $('#delete-message').text(message);
  }
  
  static setBtnStateProcessing(btnSelector) {
    // ボタンを無効化し、処理中アイコンを追加
    const element = `<span class="fa fa-spinner fa-spin" aria-hidden="true"></span>`
    $(btnSelector).prop('disabled', true).prepend(element);
  }

  static resetBtnState(btnSelector) {
    // 処理中アイコンを削除して、ボタンを有効状態に戻す
    $(btnSelector + ' .fa-spinner').remove();
    $(btnSelector).prop('disabled', false);
  }
}

//------------------------------------------
// その他
//------------------------------------------

// 雑多なメソッドを実装するクラス
class Common {
  static isEqualArrays(arr1, arr2) {
    if (arr1.length !== arr2.length) {
      return false;
    }

    for (let i = 0; i < arr1.length; i++) {
      if (arr1[i] !== arr2[i]) {
        return false;
      }
    }

    return true;
  }

  static getUrlPathNameFirst() {
    // ページのパス名を取得（例: /devmenu/job-user-management）
    const pathname = window.location.pathname;

    // パスをスラッシュで分割し2番目の要素が最初のパス名
    const segments = pathname.split('/');
    if (segments.length < 2) {
      return '';
    }
    return segments[1];
  }

  // ページネーションがクリックされたとき、次のページを決定する処理
  static getNewPage(clickedElement, currentPage, length, perPage) {
    const dataPage = clickedElement.data('page');
    let newPage;

    // 更新後のページ番号を取得
    if (clickedElement.is('#previousPage') && currentPage > 1) {
      newPage = currentPage - 1;
    } else if (clickedElement.is('#nextPage') && currentPage < Math.ceil(length / perPage)) {
      newPage = currentPage + 1;
    } else if (typeof dataPage === 'number') {
      newPage = dataPage;
    } else {
      newPage = currentPage;
    }

    return newPage;
  }
}

class History {
  static ENVIRONMENT = {
    DEV: '開発',
    PROD: '本番',
    DEFAULT: '-'
  };

  static STATUS = {
    DONE: '完了',
    FAILED: '失敗'
  };

  static HISTORY_NAME = {
    JOB_USER: {
      CREATE: 'ジョブ管理ユーザーの作成',
      PASSWORD_UPDATE: 'ジョブ管理ユーザーのパスワード変更',
      DELETE: 'ジョブ管理ユーザーの削除'
    },
    ADAPTER_AUTH_INFO: {
      CREATE: '連携アダプター認証情報の作成',
      SHOW: '連携アダプター認証情報の詳細表示',
      DELETE: '連携アダプター認証情報の削除'
    },
    CLIENT_USER: {
      CREATE: 'クライアントユーザーの作成',
      DELETE: 'クライアントユーザーの削除'
    },
    AUTH_APP_SETTING: {
      INIT: '認証アプリケーション設定の初期化'
    },
    WEB_API_ACCOUNT: {
      CREATE: 'WEB APIアカウントの作成',
      SHOW: 'Web APIアカウントの詳細表示',
      DELETE: 'Web APIアカウントの削除',
      CLIENT_SECRET: {
        UPDATE: 'クライアントシークレット更新',
        DELETE: 'クライアントシークレット削除'
      }
    },
    AZURE_MONITOR: {
      CREATE: 'Azure Monitor連携情報の作成',
      UPDATE: 'Azure Monitor連携情報の編集',
      DELETE: 'Azure Monitor連携情報の削除'
    },
    JOB_CONF: {
      IMPORT: 'ジョブ定義のインポート',
      EXPORT: 'ジョブ定義のエクスポート'
    },
    PORTAL_USER: {
      CREATE: 'ポータルユーザーの作成',
      INIT: 'ポータルユーザーの初期化',
      DELETE: 'ポータルユーザーの削除'
    }
  };

  static MESSAGE =  {
    SUCCESS: '申請履歴の作成に成功しました。',
    ERROR: '申請履歴の作成に失敗しました。'
  };

  static getUsername() {
    return $('.username').text();
  }

  static getEnvironment() {
    const menu = Common.getUrlPathNameFirst();
    switch (menu) {
      case 'devmenu':
        return History.ENVIRONMENT.DEV;
      case 'prodmenu':
        return History.ENVIRONMENT.PROD;
      default:
        return History.ENVIRONMENT.DEFAULT;
    }
  }

  // 申請履歴のレコードを作成する
  // ** historyName: HISTORY_NAMEに定義したいずれかの文字列
  // ** status: STATUSに定義したいずれかの文字列
  static add(historyName, status) {
    try {
      webapi.safeAjax({
        type: "POST",
        url: `/_api/${ENV.HISTORY_TABLE_SET_NAME}`,
        contentType: "application/json",
        data: JSON.stringify({
          [`${ENV.TABLE_LOGICAL_PREFIX}_name`]: historyName,
          [`${ENV.TABLE_LOGICAL_PREFIX}_environment`]: History.getEnvironment(),
          [`${ENV.TABLE_LOGICAL_PREFIX}_status`]: status,
          [`${ENV.TABLE_LOGICAL_PREFIX}_username`]: History.getUsername(),
        }),
        success: function (res, status, xhr) {
          console.log('申請履歴を追加しました。：', historyName);
        },
        error: function(jqXHR, textStatus, errorThrown) {
          // 失敗時の処理
          const status = jqXHR?.status;  // HTTPステータス
          const message = jqXHR?.responseJSON?.error?.message; // エラーメッセージ（innererrorの情報は含んでいないためそのまま出力しても問題なし）
          const errorCode = jqXHR?.responseJSON?.error?.code;  // エラーコード
          const innerErrorCode = jqXHR?.responseJSON?.error?.innererror?.code;  // 内部エラーコード（FJ調査用）
          const outputMessage = `${History.MESSAGE.ERROR}: (Status: ${status}, Message: "${message}", ErrorCode: ${errorCode}, InnerErrorCode: ${innerErrorCode})`;
          // ポップアップメッセージを表示
          ViewRenderer.showAlert('warning', outputMessage);
        }
      });
    } catch(error) {
      console.error("Error:History.add(): ", error);
    }
  }
}